# ------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All rights reserved.
# Licensed under the MIT License (MIT). See License.txt in the repo root for license information.
# ------------------------------------------------------------

param (
    [Parameter(Mandatory=$true)]
	[ValidateSet('install','uninstall')]
    [string] $Operation
)

function Ensure-ServiceState($targetState)
{
	Write-Host "Waiting for the service state to become $targetState"

    $svc = Get-WmiObject -Class Win32_Service -Filter "Name='ServiceFabricUpdateService'"
	while (($svc -eq $null) -or ($svc -ne $null -and $svc.State -ne $targetState))
	{
		Start-Sleep -Seconds 3
        $svc = Get-WmiObject -Class Win32_Service -Filter "Name='ServiceFabricUpdateService'"
	}

    Write-Host "The service state is $targetState as expected"
}

function Remove-Data($svcDirPath)
{
    $dataDirPath = Join-Path $svcDirPath -ChildPath "Data"
    if (Test-Path $dataDirPath)
    {
        Write-Host "Cleaning up service data"
        Remove-Item -Path $dataDirPath -Recurse -Force
    }    
}

$lastExitCode = 0
$Identity = [System.Security.Principal.WindowsIdentity]::GetCurrent()
$Principal = New-Object System.Security.Principal.WindowsPrincipal($Identity)
$IsAdmin = $Principal.IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator)

if(!$IsAdmin)
{
    Write-host "Please run the script with administrative privileges." -ForegroundColor "Red"
    exit 1
}

$service = Get-WmiObject -Class Win32_Service -Filter "Name='ServiceFabricUpdateService'"
if ($Operation -eq 'install' -and $service -ne $null)
{
    Write-host "The service is already installed." -ForegroundColor "Red"
    exit 1
}
elseif ($Operation -eq 'uninstall' -and $service -eq $null)
{
    Write-host "The service has not been installed." -ForegroundColor "Red"
    exit 1
}

$svcDirPath = $(Split-Path -parent $MyInvocation.MyCommand.Definition)
$svcExePath = Join-Path $svcDirPath -ChildPath "ServiceFabricUpdateService.exe"
if(!(Test-Path $svcExePath))
{
	Write-host "Please keep the script right beside ServiceFabricUpdateService.exe." -ForegroundColor "Red"
	exit 1
}

Write-host "Operation starts: $Operation"

if ($Operation -eq 'install')
{
    Remove-Data($svcDirPath)

    Start-Process -FilePath $svcExePath -ArgumentList "-install" -Wait
	Ensure-ServiceState('Stopped')
	Start-Service -Name "ServiceFabricUpdateService"
	Ensure-ServiceState('Running')
}
else
{
	if ($service.State -ne 'Stopped')
	{
		$service.StopService() > $null
		Ensure-ServiceState('Stopped')
	}

	$service.Delete() > $null
    
    Remove-Data($svcDirPath)
}

if($lastExitCode -ne 0)
{
    Write-host "The operation fails. Error code: $lastExitCode." -ForegroundColor "Red"
    exit 1
}
else
{
    Write-host "The operation succeeds."
    exit 0    
}
# SIG # Begin signature block
# MIIjOAYJKoZIhvcNAQcCoIIjKTCCIyUCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUZVcYQQSpbZudstArI7WqaXos
# eS6ggh38MIIE2jCCA8KgAwIBAgITMwAAAN7zAZH51QDVRgAAAAAA3jANBgkqhkiG
# 9w0BAQUFADB3MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
# A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSEw
# HwYDVQQDExhNaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EwHhcNMTgwMTMxMTkwMjU3
# WhcNMTgwOTA3MTkwMjU3WjCByjELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMRAw
# DgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24x
# LTArBgNVBAsTJE1pY3Jvc29mdCBJcmVsYW5kIE9wZXJhdGlvbnMgTGltaXRlZDEm
# MCQGA1UECxMdVGhhbGVzIFRTUyBFU046MkFENC00QjkyLUZBMDExJTAjBgNVBAMT
# HE1pY3Jvc29mdCBUaW1lLVN0YW1wIHNlcnZpY2UwggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDoJ/BjfZTYt/TcOw1wy6eQZgvIHa/pdLNc8BzT+06tREuo
# 60gQH4M2QXwPknA4dGfzUt5FM+Z1R0PzFgJ7ZUdU7RJ67GU+VyFvXbNx+daBy9NI
# MsT/jWwxejo0ueoq9a9Q9Y2TSwBwKbaEpbbynK07DgLwQzAObdYUWmHfKmMg8ZfA
# qD7R7S8r/zLI1i51XlX6gF2F2VcFHjlVdAIUPys8dQ5IhSVKLcJVAjR4FGm4Fb6b
# rq9rCOSzpJey4oMI/WV0corQVAYXAWg8EJ3OAvfUw92wAWvEaZA2pr37WFiuXP5o
# rsEhlyRlKm3N4+XPraQNyddkdWWIUP+7w/mVdKn1AgMBAAGjggEJMIIBBTAdBgNV
# HQ4EFgQUL6sYY8PwxcV4qU7hi7YxwJP3pKUwHwYDVR0jBBgwFoAUIzT42VJGcArt
# QPt2+7MrsMM1sw8wVAYDVR0fBE0wSzBJoEegRYZDaHR0cDovL2NybC5taWNyb3Nv
# ZnQuY29tL3BraS9jcmwvcHJvZHVjdHMvTWljcm9zb2Z0VGltZVN0YW1wUENBLmNy
# bDBYBggrBgEFBQcBAQRMMEowSAYIKwYBBQUHMAKGPGh0dHA6Ly93d3cubWljcm9z
# b2Z0LmNvbS9wa2kvY2VydHMvTWljcm9zb2Z0VGltZVN0YW1wUENBLmNydDATBgNV
# HSUEDDAKBggrBgEFBQcDCDANBgkqhkiG9w0BAQUFAAOCAQEAL7o4dIkSQYlQonTZ
# qPVhomP4iKYESsFPgUunh3r3Yyxt21LfPor+bo3fBoJ8wAFEabi4szRslieYDaRb
# 7vqq/XN5lwbIjAZRuUhFC8SsKocOvP0aBvW1IsxrRzu7c7OO5a1lErbcrtuaaxgm
# yL0lJ5ZoLzNF56h8LaoESth6ypQWwwls1q3P62CWdrnzIKitXwS42OVcwpnYqpwm
# 5z/w9WepI0EKDzC581O1yg/Wx4GH4WpqFo8xPaOtHIHFc+44sxNSlDJg9VmZZiY7
# +YbVUtWZGIvQenhOfzj2pf5qY8y0VoKOFjh1Ar/4HCqm8QYRKeYk0jK443xVmjpz
# cW8CsjCCBZkwggOBoAMCAQICEHmtFqFKoKWtTHNY9AcTLmUwDQYJKoZIhvcNAQEF
# BQAwXzETMBEGCgmSJomT8ixkARkWA2NvbTEZMBcGCgmSJomT8ixkARkWCW1pY3Jv
# c29mdDEtMCsGA1UEAxMkTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
# aXR5MB4XDTAxMDUwOTIzMTkyMloXDTIxMDUwOTIzMjgxM1owXzETMBEGCgmSJomT
# 8ixkARkWA2NvbTEZMBcGCgmSJomT8ixkARkWCW1pY3Jvc29mdDEtMCsGA1UEAxMk
# TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5MIICIjANBgkqhkiG
# 9w0BAQEFAAOCAg8AMIICCgKCAgEA8136gGfUWqepDCyQINA1CDx1hM23B4mcidre
# zsNg+pFoWp6UcSkYdnzC4MgldpQOWPoENDbm36/3gLrpWAsrk+WdBeN3IpH3NGQ8
# IpEdXuEJkLwU/vx1WBnhebcHkqOuiFkI2J8HygNY/GgpbTLX0qjLS/zhC0gyT+br
# uK1P5FxvE5SZ25XVdduoGreUkbR3W/VIDI9qeX0UcAR9ba+Q9dpw2Ee3v5svbOcF
# t+ERYKx5kRR8xdam5OF+1cN+5ZLSPAC1NoLeeeFt87Vu+J8zyctSfXOYNtuLoWui
# lZebo97CTSb/BpZnJQbI56zk7hIzlTGZyDUITjTKeVPVtb5jMllANsClTgRNPdtb
# BzPkWL/vP1Nk2EJZNVf9D0V8JARNntY4dBGXIpDOaER0km/VS2+whuPHNkKg0PzB
# wFr5o2G5MEdxlgoWsJHAQpXvEH8oauMqH7HkzQM/d3EExyD8SQ8dRYik18t+iK2O
# LexF28RRBMkq/OyGnpoRl1vezlOI5uK3/ayVwihA2+8EkN+BMznZskWlI4cGpVWJ
# MbsGLWAOQRh9Hy61l8sR6xXVJKWU7xUUif1Lc/oyW/zRMwD5WWJwBzLqLqtALXvK
# 3SFnGzCZjxaqI6hB0bBuEZs2xN5AdJzhWGXBYB56WzjIj7sEJnzUFkDltmtsqob9
# AL/OwTUCAwEAAaNRME8wCwYDVR0PBAQDAgHGMA8GA1UdEwEB/wQFMAMBAf8wHQYD
# VR0OBBYEFA6sgmBAVieX5SUT/CrhClOVWeSkMBAGCSsGAQQBgjcVAQQDAgEAMA0G
# CSqGSIb3DQEBBQUAA4ICAQDFEU0DOmDdXVIRd4+yuzbIsgW/tLeo2CCdXBMDthwi
# +gYTNbbIY9SaR28mV9JV8QSxJl/WqVBooLzSuG7Mw+ms3xnNeKxZdKxmNDbEGz5s
# OEwzDjASDaMm/lFTAP+vWk6EDQ8f5G0FLk6FS41sM29U0mSrv1CvfXo5oDftYwMP
# /BMGzhY21FQ7lRtRYjrlTRfUBTmSmieoW6q97Lu+4yCJYHFsVrOlE9BtDiN+lQPt
# aD3y2GO4a0226DC14cqUS/eiql2ZMLI9p8JRbCggASQnK0sAt50Ra3C+shCCvAyb
# aNCNOySHqpkocp0zX1mQvfXek546Ylo0OeKIVR25BrDBiWst12nDGRI2hNDJoNr/
# L2l4suV62uvXDMD3vWMXuDkTOKI2W3vyhVZqHWRiwTjiqr9RZqKU9RKcZiIQa/K3
# MJIt8inwPTsUQ2ii8ZwpN8vOOCAlbXxn834kEiQDCIFH7KWel/UY18+71e92lu/9
# zttWnZWgQvmXWOHXMSLTX1nmPm4iAOpDhLYl29nzCFZowGRrHXzstpOiYlduLtjn
# WI/EMUkm3d4pNYf1MHFwWxQ8ab2JEn3rLqP+2H+eglpSCivBQyvZMIifyBD7iY3m
# oYV1M35snttzE2RiaaUvfcqWbZ/4BE0wkj1uIRQhyT3gw/2Ka51K/dGhnZlDdz+w
# 2jCCBfQwggPcoAMCAQICEzMAAAECm/ALp45dw00AAAAAAQIwDQYJKoZIhvcNAQEL
# BQAwfjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcT
# B1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEoMCYGA1UE
# AxMfTWljcm9zb2Z0IENvZGUgU2lnbmluZyBQQ0EgMjAxMTAeFw0xODA3MTIyMDA4
# NDhaFw0xOTA3MjYyMDA4NDhaMHQxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNo
# aW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29y
# cG9yYXRpb24xHjAcBgNVBAMTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjCCASIwDQYJ
# KoZIhvcNAQEBBQADggEPADCCAQoCggEBANQMBgSFCJYop+lrdNDiYZPjpTBf9QjD
# W4T2wacw94hKh51p1Ph+8LLz89lGtOFkFf1vnsoWx1ioo5mM1af88Y0ad2DsXUUL
# x5tqUQmEA5LkEGqo+3I4DToEx7BWQS9MlKuwwMiiL5/R8wPZCQGlC6jdFaZeYiih
# qq7b2tVEmqMJMtnFK9rsK20RjPXRHZpzcP+a+OilunSo6nZze9WW8Bi4J8zo7ZRG
# Ebhpl5X4mLNxmDHue9i7dbU5XyJkeQ1Z6qaEOPH6j2OXrFQXusmdP2UtN5fqfnAc
# meXa4+puPtclHN8YBJ1r4EOEY28FSLG5XTwmaKwxyv1+7u7a22xGw1sCAwEAAaOC
# AXMwggFvMB8GA1UdJQQYMBYGCisGAQQBgjdMCAEGCCsGAQUFBwMDMB0GA1UdDgQW
# BBSJFwWujTtn71whoV8lsN1JuTdO7zBFBgNVHREEPjA8pDowODEeMBwGA1UECxMV
# TWljcm9zb2Z0IENvcnBvcmF0aW9uMRYwFAYDVQQFEw0yMzAwMTIrNDM3OTY0MB8G
# A1UdIwQYMBaAFEhuZOVQBdOCqhc3NyK1bajKdQKVMFQGA1UdHwRNMEswSaBHoEWG
# Q2h0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2lvcHMvY3JsL01pY0NvZFNpZ1BD
# QTIwMTFfMjAxMS0wNy0wOC5jcmwwYQYIKwYBBQUHAQEEVTBTMFEGCCsGAQUFBzAC
# hkVodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtpb3BzL2NlcnRzL01pY0NvZFNp
# Z1BDQTIwMTFfMjAxMS0wNy0wOC5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQsFAAOCAgEAk/YCcku0liV5Dhj7G7SYmGisI5nl62F8EeChIwDJfvNJKOB1/fNo
# xBnGoFZVaiK7NVUlG8FmcnOnYiK0QW6HMHosY6QmLX/5QLF4ecXD9XuNoefoQFZn
# eTUEdmAmV402GZItm9ht0EFKxUzXHcd28nqzPyUTj43bK9dPFMnCZYqjG7nf4sQY
# WAWs8e0MS+fkhH9EYstmopIqt8iRc0o8+4unKv/iYbQRWQHXi2uWIYatRm7bqplf
# sfBg6Oj4OSoSRlZ7WXt8DQL1nAIUthX8Z1v/siivkBGpOHz0RX8gH/Qrv1XJLdVj
# U8DrF6Agi5gY5Vf1ApQyhBJvOlBv9daFEue3MwPBxUcw9oeP/H+JCmIZpRNIyw8+
# X5SAVlkws9PIdPogPc4xiG5asNzG5MivUwioKlk5unq6VD0wgz1BmW3Wz9josCWk
# PNSL5itVFHo+mpE5P4x+u1eElRAmQ8JmhNwrmSPMTU8l8dBAOUnvPy3RUbI+C70q
# 1D2hb/VwqZ9rVZlBKWkSU6hL81PiyLZRN/1rvzvPrxfsgnhwoScAePRTs+HJkUnm
# wLTPm9kxZVtAlwToOAGT4gnHK/v5vwh+ApOevIRAPEvCZfzu9TYUJTVrizEkQCVU
# A6RmR7PQI7aPF8mw72ByKIQk4KgINwrunNgHHoWLnM8Un6Hmku/VdYcwggYHMIID
# 76ADAgECAgphFmg0AAAAAAAcMA0GCSqGSIb3DQEBBQUAMF8xEzARBgoJkiaJk/Is
# ZAEZFgNjb20xGTAXBgoJkiaJk/IsZAEZFgltaWNyb3NvZnQxLTArBgNVBAMTJE1p
# Y3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0eTAeFw0wNzA0MDMxMjUz
# MDlaFw0yMTA0MDMxMzAzMDlaMHcxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNo
# aW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29y
# cG9yYXRpb24xITAfBgNVBAMTGE1pY3Jvc29mdCBUaW1lLVN0YW1wIFBDQTCCASIw
# DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAJ+hbLHf20iSKnxrLhnhveLjxZlR
# I1Ctzt0YTiQP7tGn0UytdDAgEesH1VSVFUmUG0KSrphcMCbaAGvoe73siQcP9w4E
# mPCJzB/LMySHnfL0Zxws/HvniB3q506jocEjU8qN+kXPCdBer9CwQgSi+aZsk2fX
# KNxGU7CG0OUoRi4nrIZPVVIM5AMs+2qQkDBuh/NZMJ36ftaXs+ghl3740hPzCLdT
# bVK0RZCfSABKR2YRJylmqJfk0waBSqL5hKcRRxQJgp+E7VV4/gGaHVAIhQAQMEbt
# t94jRrvELVSfrx54QTF3zJvfO4OToWECtR0Nsfz3m7IBziJLVP/5BcPCIAsCAwEA
# AaOCAaswggGnMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0OBBYEFCM0+NlSRnAK7UD7
# dvuzK7DDNbMPMAsGA1UdDwQEAwIBhjAQBgkrBgEEAYI3FQEEAwIBADCBmAYDVR0j
# BIGQMIGNgBQOrIJgQFYnl+UlE/wq4QpTlVnkpKFjpGEwXzETMBEGCgmSJomT8ixk
# ARkWA2NvbTEZMBcGCgmSJomT8ixkARkWCW1pY3Jvc29mdDEtMCsGA1UEAxMkTWlj
# cm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9yaXR5ghB5rRahSqClrUxzWPQH
# Ey5lMFAGA1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9w
# a2kvY3JsL3Byb2R1Y3RzL21pY3Jvc29mdHJvb3RjZXJ0LmNybDBUBggrBgEFBQcB
# AQRIMEYwRAYIKwYBBQUHMAKGOGh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kv
# Y2VydHMvTWljcm9zb2Z0Um9vdENlcnQuY3J0MBMGA1UdJQQMMAoGCCsGAQUFBwMI
# MA0GCSqGSIb3DQEBBQUAA4ICAQAQl4rDXANENt3ptK132855UU0BsS50cVttDBOr
# zr57j7gu1BKijG1iuFcCy04gE1CZ3XpA4le7r1iaHOEdAYasu3jyi9DsOwHu4r6P
# CgXIjUji8FMV3U+rkuTnjWrVgMHmlPIGL4UD6ZEqJCJw+/b85HiZLg33B+JwvBhO
# nY5rCnKVuKE5nGctxVEO6mJcPxaYiyA/4gcaMvnMMUp2MT0rcgvI6nA9/4UKE9/C
# CmGO8Ne4F+tOi3/FNSteo7/rvH0LQnvUU3Ih7jDKu3hlXFsBFwoUDtLaFJj1PLlm
# WLMtL+f5hYbMUVbonXCUbKw5TNT2eb+qGHpiKe+imyk0BncaYsk9Hm0fgvALxyy7
# z0Oz5fnsfbXjpKh0NbhOxXEjEiZ2CzxSjHFaRkMUvLOzsE1nyJ9C/4B5IYCeFTBm
# 6EISXhrIniIh0EPpK+m79EjMLNTYMoBMJipIJF9a6lbvpt6Znco6b72BJ3QGEe52
# Ib+bgsEnVLaxaj2JoXZhtG6hE6a/qkfwEm/9ijJssv7fUciMI8lmvZ0dhxJkAj0t
# r1mPuOQh5bWwymO0eFQF1EEuUKyUsKV4q7OglnUa2ZKHE3UiLzKoCG6gW4wlv6Dv
# hMoh1useT8ma7kng9wFlb4kLfchpyOZu6qeXzjEp/w7FW1zYTRuh2Povnj8uVRZr
# yROj/TCCB3owggVioAMCAQICCmEOkNIAAAAAAAMwDQYJKoZIhvcNAQELBQAwgYgx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRt
# b25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xMjAwBgNVBAMTKU1p
# Y3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0eSAyMDExMB4XDTExMDcw
# ODIwNTkwOVoXDTI2MDcwODIxMDkwOVowfjELMAkGA1UEBhMCVVMxEzARBgNVBAgT
# Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29m
# dCBDb3Jwb3JhdGlvbjEoMCYGA1UEAxMfTWljcm9zb2Z0IENvZGUgU2lnbmluZyBQ
# Q0EgMjAxMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKvw+nIQHC6t
# 2G6qghBNNLrytlghn0IbKmvpWlCquAY4GgRJun/DDB7dN2vGEtgL8DjCmQawyDnV
# ARQxQtOJDXlkh36UYCRsr55JnOloXtLfm1OyCizDr9mpK656Ca/XllnKYBoF6WZ2
# 6DJSJhIv56sIUM+zRLdd2MQuA3WraPPLbfM6XKEW9Ea64DhkrG5kNXimoGMPLdNA
# k/jj3gcN1Vx5pUkp5w2+oBN3vpQ97/vjK1oQH01WKKJ6cuASOrdJXtjt7UORg9l7
# snuGG9k+sYxd6IlPhBryoS9Z5JA7La4zWMW3Pv4y07MDPbGyr5I4ftKdgCz1TlaR
# ITUlwzluZH9TupwPrRkjhMv0ugOGjfdf8NBSv4yUh7zAIXQlXxgotswnKDglmDlK
# Ns98sZKuHCOnqWbsYR9q4ShJnV+I4iVd0yFLPlLEtVc/JAPw0XpbL9Uj43BdD1FG
# d7P4AOG8rAKCX9vAFbO9G9RVS+c5oQ/pI0m8GLhEfEXkwcNyeuBy5yTfv0aZxe/C
# HFfbg43sTUkwp6uO3+xbn6/83bBm4sGXgXvt1u1L50kppxMopqd9Z4DmimJ4X7Iv
# hNdXnFy/dygo8e1twyiPLI9AN0/B4YVEicQJTMXUpUMvdJX3bvh4IFgsE11glZo+
# TzOE2rCIF96eTvSWsLxGoGyY0uDWiIwLAgMBAAGjggHtMIIB6TAQBgkrBgEEAYI3
# FQEEAwIBADAdBgNVHQ4EFgQUSG5k5VAF04KqFzc3IrVtqMp1ApUwGQYJKwYBBAGC
# NxQCBAweCgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMA8GA1UdEwEB/wQFMAMBAf8w
# HwYDVR0jBBgwFoAUci06AjGQQ7kUBU7h6qfHMdEjiTQwWgYDVR0fBFMwUTBPoE2g
# S4ZJaHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJvZHVjdHMvTWlj
# Um9vQ2VyQXV0MjAxMV8yMDExXzAzXzIyLmNybDBeBggrBgEFBQcBAQRSMFAwTgYI
# KwYBBQUHMAKGQmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMvTWlj
# Um9vQ2VyQXV0MjAxMV8yMDExXzAzXzIyLmNydDCBnwYDVR0gBIGXMIGUMIGRBgkr
# BgEEAYI3LgMwgYMwPwYIKwYBBQUHAgEWM2h0dHA6Ly93d3cubWljcm9zb2Z0LmNv
# bS9wa2lvcHMvZG9jcy9wcmltYXJ5Y3BzLmh0bTBABggrBgEFBQcCAjA0HjIgHQBM
# AGUAZwBhAGwAXwBwAG8AbABpAGMAeQBfAHMAdABhAHQAZQBtAGUAbgB0AC4gHTAN
# BgkqhkiG9w0BAQsFAAOCAgEAZ/KGpZjgVHkaLtPYdGcimwuWEeFjkplCln3SeQyQ
# wWVfLiw++MNy0W2D/r4/6ArKO79HqaPzadtjvyI1pZddZYSQfYtGUFXYDJJ80hpL
# HPM8QotS0LD9a+M+By4pm+Y9G6XUtR13lDni6WTJRD14eiPzE32mkHSDjfTLJgJG
# KsKKELukqQUMm+1o+mgulaAqPyprWEljHwlpblqYluSD9MCP80Yr3vw70L01724l
# ruWvJ+3Q3fMOr5kol5hNDj0L8giJ1h/DMhji8MUtzluetEk5CsYKwsatruWy2dsV
# iFFFWDgycScaf7H0J/jeLDogaZiyWYlobm+nt3TDQAUGpgEqKD6CPxNNZgvAs031
# 4Y9/HG8VfUWnduVAKmWjw11SYobDHWM2l4bf2vP48hahmifhzaWX0O5dY0HjWwec
# hz4GdwbRBrF1HxS+YWG18NzGGwS+30HHDiju3mUv7Jf2oVyW2ADWoUa9WfOXpQlL
# SBCZgB/QACnFsZulP0V3HjXG0qKin3p6IvpIlR+r+0cjgPWe+L9rt0uX4ut1eBrs
# 6jeZeRhL/9azI2h15q/6/IvrC4DqaTuv/DDtBEyO3991bWORPdGdVk5Pv4BXIqF4
# ETIheu9BCrE/+6jMpF3BoYibV3FWTkhFwELJm3ZbCoBIa/15n8G9bW1qyVJzEw16
# UM0xggSmMIIEogIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGlu
# Z3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
# cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAyMDEx
# AhMzAAABApvwC6eOXcNNAAAAAAECMAkGBSsOAwIaBQCggbowGQYJKoZIhvcNAQkD
# MQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwIwYJ
# KoZIhvcNAQkEMRYEFLOWyAz1lWlD5rYh1UHYxtsxP45/MFoGCisGAQQBgjcCAQwx
# TDBKoCSAIgBNAGkAYwByAG8AcwBvAGYAdAAgAFcAaQBuAGQAbwB3AHOhIoAgaHR0
# cDovL3d3dy5taWNyb3NvZnQuY29tL3dpbmRvd3MwDQYJKoZIhvcNAQEBBQAEggEA
# c2KvTBBtxzrWdnDhaixojWLwRXjCkpb3r4lo6V/Dm0leqD6UAL5jKF9cC/F45ao0
# 2/sLeMmCBfm2QiCDZkMGywj5CK2RsU70sBM0ay/9UE0UH+u0Acr3fG7C7vuWDpJy
# VmoPlRpfEUIDzFPY6pd9SrrK2xpGwr4WGoUDq8Sacn1O03z3RPy3+/xdVEZKAwxd
# 4wf+wMdaooqpdjuMJQwvFi0wuVExIruBgIanMvJrodcFx38Oqn/b6Lu2aZbqiOPJ
# 0SoGhfLM9PJYCUvnpKACaPgEQrIsMSBtII0N3bjqQd0LuWvsUYMwedyaTVMSPaiH
# Yst3Wgy/uwQYHn04mCcwrqGCAigwggIkBgkqhkiG9w0BCQYxggIVMIICEQIBATCB
# jjB3MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
# UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSEwHwYDVQQD
# ExhNaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0ECEzMAAADe8wGR+dUA1UYAAAAAAN4w
# CQYFKw4DAhoFAKBdMBgGCSqGSIb3DQEJAzELBgkqhkiG9w0BBwEwHAYJKoZIhvcN
# AQkFMQ8XDTE4MDgxMTAzNDEzN1owIwYJKoZIhvcNAQkEMRYEFPKz3pIFckVGopzi
# s7+ShYR8YG0cMA0GCSqGSIb3DQEBBQUABIIBAEfrw05za7CTHrnomRMJxlAcDBEr
# FmLZEKOv2hiKsJIZcABwwYuUtJoEg+fBe5XUDfsRv5h0cntAzt4woux6FcpSlwQ5
# dzc0TIlICiVQx7bSKzqgf9FEc5pHJIw18xvv+Jz1B3WurBaGeOrO687q/ImsG/+J
# MO1pEo4Z9lxzelQqmNe68f/NibkLinUoqXRT6H555LsPo7V+cZlGQDR3VOP46TUC
# +WjFPtrlt3lrYCIN+jEUyMkAMBe8E7qVhg9d0N2Xqc0QBZ76B8f5/75FhW4uzIrw
# C0q6W1PjA1Pb77SpM0XD3J4a/QoQeHvYywUKJxe0lBGuVhYueublJ9ogNRo=
# SIG # End signature block
